@@ -1,3 +1,6 @@
+use serde::de::DeserializeOwned;
+use serde::Serialize;
+
 use crate::checkout::Checkout;
 use crate::context::Context;
 use crate::customer::{Address, Contact};
@@ -7,24 +10,21 @@ use crate::money::Currency;
 use crate::payment::PaymentProcessor;
 use crate::shipping::FulfillmentTypeSelection;
 
-pub async fn create_handler<C: Context + Send>(
-    ctx: &mut C,
+pub async fn create_handler<T: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
+    ctx: &mut T,
     currency: Currency,
     contact: Option<Contact>,
     shipping_address: Option<Address>,
-) -> Result<Checkout<C>, Error> {
-    let mut co = Checkout::new(ctx, currency, contact, shipping_address).await?;
-    co.populate_associated_fields(ctx).await?;
-
-    Ok(co)
+) -> Result<Checkout<P>, Error> {
+    Checkout::new(ctx, currency, contact, shipping_address).await
 }
 
-pub async fn add_item_handler<C: Context + Send>(
-    ctx: &mut C,
+pub async fn add_item_handler<T: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
+    ctx: &mut T,
     checkout_id: &str,
     sku: String,
     quantity: u64,
-) -> Result<Checkout<C>, Error> {
+) -> Result<Checkout<P>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -35,17 +35,19 @@ pub async fn add_item_handler<C: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.add_item(ctx, sku, quantity).await?;
-    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn remove_item_handler<C: Context + Send>(
-    ctx: &mut C,
+pub async fn remove_item_handler<
+    T: Context + Send,
+    P: Sync + Send + Serialize + DeserializeOwned,
+>(
+    ctx: &mut T,
     checkout_id: &str,
     sku: String,
     quantity: u64,
-) -> Result<Checkout<C>, Error> {
+) -> Result<Checkout<P>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -56,15 +58,17 @@ pub async fn remove_item_handler<C: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.remove_item(ctx, sku, quantity).await?;
-    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn confirm_items_handler<C: Context + Send>(
-    ctx: &mut C,
+pub async fn confirm_items_handler<
+    T: Context + Send,
+    P: Sync + Send + Serialize + DeserializeOwned,
+>(
+    ctx: &mut T,
     checkout_id: &str,
-) -> Result<Checkout<C>, Error> {
+) -> Result<Checkout<P>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -75,16 +79,18 @@ pub async fn confirm_items_handler<C: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.confirm_items(ctx).await?;
-    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn update_contact_handler<C: Context + Send>(
-    ctx: &mut C,
+pub async fn update_contact_handler<
+    T: Context + Send,
+    P: Sync + Send + Serialize + DeserializeOwned,
+>(
+    ctx: &mut T,
     checkout_id: &str,
     contact: Contact,
-) -> Result<Checkout<C>, Error> {
+) -> Result<Checkout<P>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -95,16 +101,18 @@ pub async fn update_contact_handler<C: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.update_contact(ctx, contact).await?;
-    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn update_shipping_address_handler<C: Context + Send>(
-    ctx: &mut C,
+pub async fn update_shipping_address_handler<
+    T: Context + Send,
+    P: Sync + Send + Serialize + DeserializeOwned,
+>(
+    ctx: &mut T,
     checkout_id: &str,
     shipping_address: Address,
-) -> Result<Checkout<C>, Error> {
+) -> Result<Checkout<P>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -115,16 +123,18 @@ pub async fn update_shipping_address_handler<C: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.update_shipping_address(ctx, shipping_address).await?;
-    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn update_fulfillment_type_handler<C: Context + Send>(
-    ctx: &mut C,
+pub async fn update_fulfillment_type_handler<
+    T: Context + Send,
+    P: Sync + Send + Serialize + DeserializeOwned,
+>(
+    ctx: &mut T,
     checkout_id: &str,
     fulfillment_type: FulfillmentTypeSelection,
-) -> Result<Checkout<C>, Error> {
+) -> Result<Checkout<P>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -135,16 +145,18 @@ pub async fn update_fulfillment_type_handler<C: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.update_fulfillment_type(ctx, fulfillment_type).await?;
-    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn initiate_payment_handler<C: Context + Send>(
-    ctx: &mut C,
+pub async fn initiate_payment_handler<
+    T: Context + Send,
+    P: Sync + Send + Serialize + DeserializeOwned,
+>(
+    ctx: &mut T,
     checkout_id: &str,
-    args: <C as PaymentProcessor>::ProcessArgs,
-) -> Result<Checkout<C>, Error> {
+    args: <T as PaymentProcessor>::ProcessArgs,
+) -> Result<Checkout<P>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -155,17 +167,19 @@ pub async fn initiate_payment_handler<C: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.initiate_payment(ctx, args).await?;
-    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn initiate_order_handler<C: Context + Send>(
+pub async fn initiate_order_handler<
+    T: Context + Send,
+    P: Sync + Send + Serialize + DeserializeOwned,
+>(
     internal_ctx: &InternalContext,
-    ctx: &mut C,
+    ctx: &mut T,
     checkout_id: &str,
-    args: <C as PaymentProcessor>::ProcessArgs,
-) -> Result<Checkout<C>, Error> {
+    args: <T as PaymentProcessor>::ProcessArgs,
+) -> Result<Checkout<P>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -176,7 +190,6 @@ pub async fn initiate_order_handler<C: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     co.initiate_order(internal_ctx, ctx, args).await?;
-    co.populate_associated_fields(ctx).await?;
 
     Ok(co)
 }
