@@ -1,6 +1,7 @@
 use async_trait::async_trait;
 use chrono::{DateTime, Utc};
 use schemars::JsonSchema;
+use serde::de::DeserializeOwned;
 use serde::{Deserialize, Serialize};
 use uuid::Uuid;
 
@@ -13,7 +14,7 @@ use crate::invoice::Invoice;
 use crate::item::Item;
 use crate::money::{Currency, Money};
 use crate::order::Order;
-use crate::payment::{handlers as payment_handlers, Payment};
+use crate::payment::handlers as payment_handlers;
 use crate::payment::{Initiator, PaymentProcessor};
 use crate::shipping::{FulfillmentType, FulfillmentTypeSelection, ShippingQuote};
 
@@ -28,7 +29,7 @@ pub enum State {
 }
 
 #[derive(Serialize, Deserialize, JsonSchema)]
-pub struct Checkout<C: Context + Send> {
+pub struct Checkout<P: Sync + Send> {
     pub id: String,
     pub state: State,
     pub currency: Currency,
@@ -42,33 +43,32 @@ pub struct Checkout<C: Context + Send> {
     pub payment_id: Option<String>,
     pub order_id: Option<String>,
     // associations
-    pub payment: Option<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
-    pub order: Option<Order<C>>,
+    pub payment: Option<P>,
 }
 
 #[async_trait]
 pub trait CheckoutStore {
-    async fn create_checkout<C: Context + Send>(&mut self, co: &Checkout<C>) -> Result<(), Error>;
-    async fn update_checkout<C: Context + Send>(&mut self, co: &Checkout<C>) -> Result<(), Error>;
-    async fn get_checkout<C: Context + Send>(
+    async fn create_checkout<P: Sync + Send + Serialize + DeserializeOwned>(
+        &mut self,
+        co: &Checkout<P>,
+    ) -> Result<(), Error>;
+    async fn update_checkout<P: Sync + Send + Serialize + DeserializeOwned>(
+        &mut self,
+        co: &Checkout<P>,
+    ) -> Result<(), Error>;
+    async fn get_checkout<P: Sync + Send + Serialize + DeserializeOwned>(
         &mut self,
         id: &str,
-    ) -> Result<Option<Checkout<C>>, Error>;
-    async fn get_checkout_for_update<C: Context + Send>(
+    ) -> Result<Option<Checkout<P>>, Error>;
+    async fn get_checkout_for_update<P: Sync + Send + Serialize + DeserializeOwned>(
         &mut self,
         id: &str,
-    ) -> Result<Option<Checkout<C>>, Error>;
+    ) -> Result<Option<Checkout<P>>, Error>;
 }
 
 // constructor
-impl<C: Context + Send> Checkout<C> {
-    pub async fn new(
+impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
+    pub async fn new<C: Context + Send>(
         ctx: &mut C,
         currency: Currency,
         contact: Option<Contact>,
@@ -89,7 +89,6 @@ impl<C: Context + Send> Checkout<C> {
             order_id: None,
             // associations
             payment: None,
-            order: None,
         };
 
         ctx.create_checkout(&co).await?;
@@ -99,14 +98,17 @@ impl<C: Context + Send> Checkout<C> {
 }
 
 // state transitions
-impl<C: Context + Send> Checkout<C> {
-    async fn enter_shopping_state(&mut self, ctx: &mut C) -> Result<(), Error> {
+impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
+    async fn enter_shopping_state<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
         self.reset_shipping_options();
         ctx.free_items(&self.items).await?;
         Ok(self.state = State::Shopping)
     }
 
-    async fn enter_items_confirmed_state(&mut self, ctx: &mut C) -> Result<(), Error> {
+    async fn enter_items_confirmed_state<C: Context + Send>(
+        &mut self,
+        ctx: &mut C,
+    ) -> Result<(), Error> {
         self.reset_shipping_options();
         match ctx.reserve_items(&self.items).await? {
             ReserveResult::Success(deadline) => {
@@ -132,7 +134,7 @@ impl<C: Context + Send> Checkout<C> {
         }
     }
 
-    async fn enter_payment_in_progress_state(
+    async fn enter_payment_in_progress_state<C: Context + Send>(
         &mut self,
         ctx: &mut C,
         args: <C as PaymentProcessor>::ProcessArgs,
@@ -154,8 +156,8 @@ impl<C: Context + Send> Checkout<C> {
 }
 
 // private methods
-impl<C: Context + Send> Checkout<C> {
-    async fn ensure_shopping_state(&mut self, ctx: &mut C) -> Result<(), Error> {
+impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
+    async fn ensure_shopping_state<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
         match self.state {
             State::Shopping => Ok(()),
             State::ItemsConfirmed(_) => self.enter_shopping_state(ctx).await,
@@ -165,7 +167,10 @@ impl<C: Context + Send> Checkout<C> {
         }
     }
 
-    async fn ensure_items_confirmed_state(&mut self, ctx: &mut C) -> Result<(), Error> {
+    async fn ensure_items_confirmed_state<C: Context + Send>(
+        &mut self,
+        ctx: &mut C,
+    ) -> Result<(), Error> {
         match self.state {
             State::ItemsConfirmed(_) => Ok(()),
             State::Shopping => self.enter_items_confirmed_state(ctx).await,
@@ -175,7 +180,7 @@ impl<C: Context + Send> Checkout<C> {
         }
     }
 
-    async fn ensure_payment_in_progress_state(
+    async fn ensure_payment_in_progress_state<C: Context + Send>(
         &mut self,
         ctx: &mut C,
         args: <C as PaymentProcessor>::ProcessArgs,
@@ -237,11 +242,11 @@ impl<C: Context + Send> Checkout<C> {
         }
     }
 
-    async fn update_invoice(&mut self, ctx: &mut C) -> Result<(), Error> {
+    async fn update_invoice<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
         Ok(self.invoice = Some(ctx.generate_invoice(self).await?))
     }
 
-    async fn update_item_prices(&mut self, ctx: &mut C) -> Result<(), Error> {
+    async fn update_item_prices<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
         let item_prices = ctx
             .calculate_item_prices(&self.currency, &self.promo_codes, &self.items)
             .await?;
@@ -266,8 +271,13 @@ impl<C: Context + Send> Checkout<C> {
 }
 
 // public methods
-impl<C: Context + Send> Checkout<C> {
-    pub async fn add_item(&mut self, ctx: &mut C, sku: String, quantity: u64) -> Result<(), Error> {
+impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
+    pub async fn add_item<C: Context + Send>(
+        &mut self,
+        ctx: &mut C,
+        sku: String,
+        quantity: u64,
+    ) -> Result<(), Error> {
         self.ensure_shopping_state(ctx).await?;
 
         let mut is_added = false;
@@ -297,7 +307,7 @@ impl<C: Context + Send> Checkout<C> {
         ctx.update_checkout(self).await
     }
 
-    pub async fn remove_item(
+    pub async fn remove_item<C: Context + Send>(
         &mut self,
         ctx: &mut C,
         sku: String,
@@ -328,19 +338,23 @@ impl<C: Context + Send> Checkout<C> {
         ctx.update_checkout(self).await
     }
 
-    pub async fn confirm_items(&mut self, ctx: &mut C) -> Result<(), Error> {
+    pub async fn confirm_items<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
         self.ensure_items_confirmed_state(ctx).await?;
 
         ctx.update_checkout(self).await
     }
 
-    pub async fn update_contact(&mut self, ctx: &mut C, contact: Contact) -> Result<(), Error> {
+    pub async fn update_contact<C: Context + Send>(
+        &mut self,
+        ctx: &mut C,
+        contact: Contact,
+    ) -> Result<(), Error> {
         self.contact = Some(contact);
 
         ctx.update_checkout(self).await
     }
 
-    pub async fn update_shipping_address(
+    pub async fn update_shipping_address<C: Context + Send>(
         &mut self,
         ctx: &mut C,
         address: Address,
@@ -362,7 +376,7 @@ impl<C: Context + Send> Checkout<C> {
         ctx.update_checkout(self).await
     }
 
-    pub async fn update_fulfillment_type(
+    pub async fn update_fulfillment_type<C: Context + Send>(
         &mut self,
         ctx: &mut C,
         fulfillment_type: FulfillmentTypeSelection,
@@ -396,7 +410,7 @@ impl<C: Context + Send> Checkout<C> {
         ctx.update_checkout(self).await
     }
 
-    pub async fn initiate_payment(
+    pub async fn initiate_payment<C: Context + Send>(
         &mut self,
         ctx: &mut C,
         args: <C as PaymentProcessor>::ProcessArgs,
@@ -406,7 +420,7 @@ impl<C: Context + Send> Checkout<C> {
         ctx.update_checkout(self).await
     }
 
-    pub async fn initiate_order(
+    pub async fn initiate_order<C: Context + Send>(
         &mut self,
         internal_ctx: &InternalContext,
         ctx: &mut C,
@@ -426,7 +440,7 @@ impl<C: Context + Send> Checkout<C> {
         ctx.update_checkout(self).await
     }
 
-    pub async fn revert(&mut self, ctx: &mut C) -> Result<(), Error> {
+    pub async fn revert<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
         match self.state {
             State::Shopping => Ok(()),
             State::ItemsConfirmed(_) | State::PaymentInProgress(_) => {
@@ -446,7 +460,7 @@ impl<C: Context + Send> Checkout<C> {
         }
     }
 
-    pub async fn complete(
+    pub async fn complete<C: Context + Send>(
         &mut self,
         internal_ctx: &InternalContext,
         ctx: &mut C,
@@ -463,22 +477,4 @@ impl<C: Context + Send> Checkout<C> {
 
         ctx.update_checkout(self).await
     }
-
-    // utility method to be called by handlers
-    pub async fn populate_associated_fields(&mut self, ctx: &mut C) -> Result<(), Error> {
-        if let Some(ref payment_id) = self.payment_id {
-            self.payment = ctx.get_payment(payment_id).await?;
-        }
-
-        if let Some(ref order_id) = self.order_id {
-            let mut order = ctx.get_order(order_id).await?;
-            if let Some(ref mut od) = order {
-                od.populate_associated_fields(ctx).await?;
-            }
-
-            self.order = order;
-        }
-
-        Ok(())
-    }
 }
