@@ -17,7 +17,7 @@ pub async fn confirm_handler<C: Context + Send>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
     order_id: &str,
-) -> Result<Order<C>, Error> {
+) -> Result<Order, Error> {
     let maybe_od = ctx.get_order_for_update(&order_id).await?;
     if maybe_od.is_none() {
         return Err(new_not_found_error(
@@ -35,7 +35,7 @@ pub async fn cancel_handler<C: Context + Send>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
     order_id: &str,
-) -> Result<Order<C>, Error> {
+) -> Result<Order, Error> {
     // first retrieve the associated payment for update to avoid deadlock
     let maybe_payment = get_order_payment_for_update(ctx, order_id).await?;
 
@@ -56,7 +56,19 @@ pub async fn cancel_handler<C: Context + Send>(
         }
 
         let mut payment = maybe_payment.unwrap();
-        cancel_payment(internal_ctx, ctx, &mut payment, Initiator::Order(&mut od)).await?;
+        cancel_payment(
+            internal_ctx,
+            ctx,
+            &mut payment,
+            Initiator::<
+                Payment<
+                    <C as PaymentProcessor>::PrivateData,
+                    <C as PaymentProcessor>::PublicData,
+                    <C as PaymentProcessor>::ProcessArgs,
+                >,
+            >::Order(&mut od),
+        )
+        .await?;
     } else {
         od.cancel(internal_ctx, ctx).await?;
     }
@@ -68,7 +80,7 @@ pub async fn reconcile_handler<C: Context + Send>(
     ctx: &mut C,
     lookup_key: LookupKey,
 ) -> Result<(), Error> {
-    let maybe_order: Option<Order<C>> = match &lookup_key {
+    let maybe_order = match &lookup_key {
         LookupKey::CheckoutId(checkout_id) => get_order_by_checkout_id(ctx, checkout_id).await?,
         LookupKey::OrderId(order_id) => ctx.get_order_for_update(order_id).await?,
     };
@@ -88,8 +100,16 @@ pub async fn reconcile_handler<C: Context + Send>(
 async fn get_order_by_checkout_id<C: Context + Send>(
     ctx: &mut C,
     checkout_id: &str,
-) -> Result<Option<Order<C>>, Error> {
-    let maybe_checkout: Option<Checkout<C>> = ctx.get_checkout_for_update(checkout_id).await?;
+) -> Result<Option<Order>, Error> {
+    let maybe_checkout: Option<
+        Checkout<
+            Payment<
+                <C as PaymentProcessor>::PrivateData,
+                <C as PaymentProcessor>::PublicData,
+                <C as PaymentProcessor>::ProcessArgs,
+            >,
+        >,
+    > = ctx.get_checkout_for_update(checkout_id).await?;
     if maybe_checkout.is_none() {
         return Err(new_not_found_error(
             "the associated checkout could not be found",
@@ -119,7 +139,7 @@ async fn get_order_payment_for_update<C: Context + Send>(
     >,
     Error,
 > {
-    let maybe_od: Option<Order<C>> = ctx.get_order(order_id).await?;
+    let maybe_od = ctx.get_order(order_id).await?;
     if maybe_od.is_none() {
         return Err(new_not_found_error(
             "the requested order could not be found",
