@@ -21,7 +21,7 @@ use crate::{
 pub mod handlers;
 
 #[derive(Serialize, Deserialize, JsonSchema)]
-pub struct Order<C: Context + Send> {
+pub struct Order {
     pub id: String,
     pub state: OrderState,
     pub state_reconciled: bool,
@@ -31,14 +31,6 @@ pub struct Order<C: Context + Send> {
     pub fulfillment_type: FulfillmentType,
     pub invoice: Invoice,
     pub payment_id: Option<String>,
-    // associations
-    pub payment: Option<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
 }
 
 #[derive(Serialize, Deserialize, JsonSchema)]
@@ -70,51 +62,42 @@ impl cloud_pubsub::FromPubSubMessage for LookupKey {
 
 #[async_trait]
 pub trait OrderStore {
-    async fn create_order<C: Context + Send>(&mut self, order: &Order<C>) -> Result<(), Error>;
-    async fn update_order<C: Context + Send>(&mut self, order: &Order<C>) -> Result<(), Error>;
-    async fn get_order_for_update<C: Context + Send>(
-        &mut self,
-        id: &str,
-    ) -> Result<Option<Order<C>>, Error>;
-    async fn get_order<C: Context + Send>(&mut self, id: &str) -> Result<Option<Order<C>>, Error>;
+    async fn create_order(&mut self, order: &Order) -> Result<(), Error>;
+    async fn update_order(&mut self, order: &Order) -> Result<(), Error>;
+    async fn get_order_for_update(&mut self, id: &str) -> Result<Option<Order>, Error>;
+    async fn get_order(&mut self, id: &str) -> Result<Option<Order>, Error>;
 }
 
 #[async_trait]
 pub trait OrderEvents {
-    async fn on_payment_in_progress<C: Context + Send>(
-        &mut self,
-        _order: &Order<C>,
-    ) -> Result<(), Error> {
+    async fn on_payment_in_progress(&mut self, _order: &Order) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_pending_confirmation<C: Context + Send>(
-        &mut self,
-        _order: &Order<C>,
-    ) -> Result<(), Error> {
+    async fn on_pending_confirmation(&mut self, _order: &Order) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_confirmed<C: Context + Send>(&mut self, _order: &Order<C>) -> Result<(), Error> {
+    async fn on_confirmed(&mut self, _order: &Order) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_fulfilled<C: Context + Send>(&mut self, _order: &Order<C>) -> Result<(), Error> {
+    async fn on_fulfilled(&mut self, _order: &Order) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_cancelled<C: Context + Send>(&mut self, _order: &Order<C>) -> Result<(), Error> {
+    async fn on_cancelled(&mut self, _order: &Order) -> Result<(), Error> {
         Ok(())
     }
 }
 
-impl<C: Context + Send> Order<C> {
-    pub async fn new_from_checkout(
+impl Order {
+    pub async fn new_from_checkout<C: Context + Send, P: Sync + Send>(
         internal_ctx: &InternalContext,
         ctx: &mut C,
-        checkout: &Checkout<C>,
+        checkout: &Checkout<P>,
     ) -> Result<Self, Error> {
-        let order = Self {
+        let order = Order {
             id: Uuid::new_v4().to_string(),
             state_reconciled: false,
             state: OrderState::PaymentInProgress,
@@ -124,7 +107,6 @@ impl<C: Context + Send> Order<C> {
             fulfillment_type: checkout.fulfillment_type.as_ref().unwrap().clone(),
             invoice: checkout.invoice.as_ref().unwrap().clone(),
             payment_id: checkout.payment_id.clone(),
-            payment: None,
         };
 
         ctx.create_order(&order).await?;
@@ -135,10 +117,10 @@ impl<C: Context + Send> Order<C> {
         Ok(order)
     }
 
-    pub async fn new_from_checkout_missing_payment(
+    pub async fn new_from_checkout_missing_payment<C: Context + Send, P: Sync + Send>(
         internal_ctx: &InternalContext,
         ctx: &mut C,
-        checkout: &Checkout<C>,
+        checkout: &Checkout<P>,
         payment_args: <C as PaymentProcessor>::ProcessArgs,
     ) -> Result<Self, Error> {
         let mut order = Order {
@@ -151,7 +133,6 @@ impl<C: Context + Send> Order<C> {
             fulfillment_type: checkout.fulfillment_type.as_ref().unwrap().clone(),
             invoice: checkout.invoice.as_ref().unwrap().clone(),
             payment_id: None,
-            payment: None,
         };
 
         ctx.create_order(&order).await?;
@@ -159,7 +140,13 @@ impl<C: Context + Send> Order<C> {
         let charge_amount = order.invoice.initial_charge_amount.clone();
         let payment = payment_handlers::create_handler(
             ctx,
-            Initiator::Order(&mut order),
+            Initiator::<
+                Payment<
+                    <C as PaymentProcessor>::PrivateData,
+                    <C as PaymentProcessor>::PublicData,
+                    <C as PaymentProcessor>::ProcessArgs,
+                >,
+            >::Order(&mut order),
             charge_amount,
             payment_args,
         )
@@ -176,7 +163,7 @@ impl<C: Context + Send> Order<C> {
     }
 }
 
-impl<C: Context + Send> Order<C> {
+impl Order {
     async fn assert_reconciled(&self) -> Result<(), Error> {
         match self.state_reconciled {
             true => Ok(()),
@@ -265,8 +252,8 @@ impl<C: Context + Send> Order<C> {
     }
 }
 
-impl<C: Context + Send> Order<C> {
-    pub async fn prompt_confirmation(
+impl Order {
+    pub async fn prompt_confirmation<C: Context + Send>(
         &mut self,
         internal_ctx: &InternalContext,
         ctx: &mut C,
@@ -275,7 +262,7 @@ impl<C: Context + Send> Order<C> {
         ctx.update_order(self).await
     }
 
-    pub async fn confirm(
+    pub async fn confirm<C: Context + Send>(
         &mut self,
         internal_ctx: &InternalContext,
         ctx: &mut C,
@@ -284,7 +271,7 @@ impl<C: Context + Send> Order<C> {
         ctx.update_order(self).await
     }
 
-    pub async fn fulfill(
+    pub async fn fulfill<C: Context + Send>(
         &mut self,
         internal_ctx: &InternalContext,
         ctx: &mut C,
@@ -293,7 +280,7 @@ impl<C: Context + Send> Order<C> {
         ctx.update_order(self).await
     }
 
-    pub async fn cancel(
+    pub async fn cancel<C: Context + Send>(
         &mut self,
         internal_ctx: &InternalContext,
         ctx: &mut C,
@@ -302,7 +289,7 @@ impl<C: Context + Send> Order<C> {
         ctx.update_order(self).await
     }
 
-    pub async fn reconcile(&mut self, ctx: &mut C) -> Result<(), Error> {
+    pub async fn reconcile<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
         if self.state_reconciled == true {
             return Ok(());
         }
@@ -318,15 +305,6 @@ impl<C: Context + Send> Order<C> {
         self.state_reconciled = true;
         ctx.update_order(self).await
     }
-
-    // utility method to be called by handlers
-    pub async fn populate_associated_fields(&mut self, ctx: &mut C) -> Result<(), Error> {
-        if let Some(ref payment_id) = self.payment_id {
-            self.payment = ctx.get_payment(payment_id).await?;
-        }
-
-        Ok(())
-    }
 }
 
 async fn publish_state_change_event(
