@@ -1,3 +1,7 @@
+use serde::de::DeserializeOwned;
+use serde::Serialize;
+
+use crate::checkout::Checkout;
 use crate::context::Context;
 use crate::error::{new_not_found_error, Error};
 use crate::internal_context::InternalContext;
@@ -7,9 +11,13 @@ use super::{
     cancel, new, process, queue_for_processing, Initiator, InitiatorId, Payment, PaymentProcessor,
 };
 
-pub async fn create_handler<'a, C: Context + Send>(
+pub async fn create_handler<
+    'a,
+    C: Context + Send,
+    P: Sync + Send + Serialize + DeserializeOwned,
+>(
     ctx: &mut C,
-    initiator: Initiator<'a, C>,
+    initiator: Initiator<'a, P>,
     charge_amount: Money,
     args: <C as PaymentProcessor>::ProcessArgs,
 ) -> Result<
@@ -90,7 +98,15 @@ pub async fn process_handler<C: Context + Send>(
 
     match &payment.initiator_id {
         InitiatorId::Checkout(co_id) => {
-            let maybe_co = ctx.get_checkout_for_update(co_id).await?;
+            let maybe_co: Option<
+                Checkout<
+                    Payment<
+                        <C as PaymentProcessor>::PrivateData,
+                        <C as PaymentProcessor>::PublicData,
+                        <C as PaymentProcessor>::ProcessArgs,
+                    >,
+                >,
+            > = ctx.get_checkout_for_update(co_id).await?;
             if maybe_co.is_none() {
                 // this isn't supposed to happen.
                 return Err(new_not_found_error(
@@ -121,7 +137,13 @@ pub async fn process_handler<C: Context + Send>(
                 internal_ctx,
                 ctx,
                 &mut payment,
-                Initiator::Order(&mut order),
+                Initiator::<
+                    Payment<
+                        <C as PaymentProcessor>::PrivateData,
+                        <C as PaymentProcessor>::PublicData,
+                        <C as PaymentProcessor>::ProcessArgs,
+                    >,
+                >::Order(&mut order),
             )
             .await
         }
@@ -148,7 +170,15 @@ pub async fn cancel_handler<C: Context + Send>(
 
     match &payment.initiator_id {
         InitiatorId::Checkout(co_id) => {
-            let maybe_co = ctx.get_checkout_for_update(co_id).await?;
+            let maybe_co: Option<
+                Checkout<
+                    Payment<
+                        <C as PaymentProcessor>::PrivateData,
+                        <C as PaymentProcessor>::PublicData,
+                        <C as PaymentProcessor>::ProcessArgs,
+                    >,
+                >,
+            > = ctx.get_checkout_for_update(co_id).await?;
             if maybe_co.is_none() {
                 return Err(new_not_found_error("the checkout could not be found"));
             }
@@ -158,7 +188,13 @@ pub async fn cancel_handler<C: Context + Send>(
                 internal_ctx,
                 ctx,
                 &mut payment,
-                Initiator::Checkout(&mut checkout),
+                Initiator::<
+                    Payment<
+                        <C as PaymentProcessor>::PrivateData,
+                        <C as PaymentProcessor>::PublicData,
+                        <C as PaymentProcessor>::ProcessArgs,
+                    >,
+                >::Checkout(&mut checkout),
             )
             .await?;
         }
@@ -173,7 +209,13 @@ pub async fn cancel_handler<C: Context + Send>(
                 internal_ctx,
                 ctx,
                 &mut payment,
-                Initiator::Order(&mut order),
+                Initiator::<
+                    Payment<
+                        <C as PaymentProcessor>::PrivateData,
+                        <C as PaymentProcessor>::PublicData,
+                        <C as PaymentProcessor>::ProcessArgs,
+                    >,
+                >::Order(&mut order),
             )
             .await?;
         }
